@@ -26,6 +26,12 @@ $(function ()
         saveProfileDrivers(name, true);
     });
 
+    // Handle driver source change
+    $("#driver_source").change(function ()
+    {
+        onDriverSourceChange();
+    });
+
     // Restart
     $('#restart_sm').confirmation({
         onConfirm: function ()
@@ -166,11 +172,29 @@ function saveProfileInfo()
     //var url     =  "/api/profiles/" + name + "/" + port + "/" + autostart;
     var url = "/api/profiles/" + name;
 
+    // Parse scripts back to object to avoid double JSON encoding
+    var scripts = window.currentProfileScripts || "";
+    var scriptsObj = "";
+    if (scripts && scripts !== "")
+    {
+        try
+        {
+            // If scripts is a JSON string, parse it to an object
+            scriptsObj = JSON.parse(scripts);
+        }
+        catch (e)
+        {
+            // If parsing fails, keep it as is
+            scriptsObj = scripts;
+        }
+    }
+
     var profileInfo = {
         "port": port,
         "autostart": autostart,
         "autoconnect": autoconnect,
-        "driver_source": driver_source
+        "driver_source": driver_source,
+        "scripts": scriptsObj
     };
     profileInfo = JSON.stringify(profileInfo);
     console.log("Profile info " + profileInfo);
@@ -247,22 +271,32 @@ function loadCurrentProfileDrivers()
     clearDriverSelection();
 
     var name = $("#profiles option:selected").text();
-    var url = "/api/profiles/" + name + "/labels";
 
-    $.getJSON(url, function (drivers)
+    // First load profile data to get the driver source
+    loadProfileData(function ()
     {
-        $.each(drivers, function (i, driver)
+        // After profile data is loaded, load drivers from the correct source
+        var driverSource = $("#driver_source").val() || 'system';
+        loadDriversFromSource(driverSource, function ()
         {
-            var label = driver.label;
-            //console.log("Driver label is " + label);
-            var selector = "#drivers_list [value='" + label + "']";
-            $(selector).prop('selected', true);
-        });
+            // Then select the profile's drivers
+            var url = "/api/profiles/" + name + "/labels";
+            $.getJSON(url, function (drivers)
+            {
+                $.each(drivers, function (i, driver)
+                {
+                    var label = driver.label;
+                    //console.log("Driver label is " + label);
+                    var selector = "#drivers_list [value='" + label + "']";
+                    $(selector).prop('selected', true);
+                });
 
-        $("#drivers_list").selectpicker('refresh');
+                $("#drivers_list").selectpicker('refresh');
+            });
+        });
     });
 
-    url = encodeURI("/api/profiles/" + name + "/remote");
+    var url = encodeURI("/api/profiles/" + name + "/remote");
 
     $.getJSON(url, function (data)
     {
@@ -275,12 +309,9 @@ function loadCurrentProfileDrivers()
             $("#remote_drivers").val("");
         }
     });
-
-    loadProfileData();
-
 }
 
-function loadProfileData()
+function loadProfileData(callback)
 {
     var name = $("#profiles option:selected").text();
     var url = encodeURI("/api/profiles/" + name);
@@ -308,6 +339,14 @@ function loadProfileData()
             $("#driver_source").val("system");
         }
 
+        // Store scripts for later use (to preserve when updating profile)
+        window.currentProfileScripts = info.scripts || "";
+
+        // Call callback if provided
+        if (callback && typeof callback === 'function')
+        {
+            callback();
+        }
     });
 }
 
@@ -512,38 +551,90 @@ function restartDriver(label)
     });
 }
 
+// Load drivers from specific source
+function loadDriversFromSource(source, callback)
+{
+    var url = "/api/drivers?source=" + encodeURIComponent(source);
+
+    $.getJSON(url, function (drivers)
+    {
+        // Clear current driver list
+        $("#drivers_list").empty();
+
+        // Group drivers by family
+        var families = {};
+        $.each(drivers, function (i, driver)
+        {
+            if (!families[driver.family])
+            {
+                families[driver.family] = [];
+            }
+            families[driver.family].push(driver);
+        });
+
+        // Add drivers to select list grouped by family
+        $.each(Object.keys(families).sort(), function (i, family)
+        {
+            var optgroup = $('<optgroup>').attr('label', family);
+            $.each(families[family], function (j, driver)
+            {
+                optgroup.append($('<option>').attr('value', driver.label).text(driver.label));
+            });
+            $("#drivers_list").append(optgroup);
+        });
+
+        // Refresh the selectpicker
+        $("#drivers_list").selectpicker('refresh');
+
+        console.log("Loaded " + drivers.length + " drivers from source: " + source);
+
+        // Call callback if provided
+        if (callback && typeof callback === 'function')
+        {
+            callback();
+        }
+    }).fail(function ()
+    {
+        console.error('Failed to load drivers from source: ' + source);
+        alert('Failed to load drivers from ' + source + '. Please check the logs.');
+    });
+}
+
 // Detect available KStars flatpaks for driver sources
 function detectFlatpaks()
 {
-    $.getJSON("/api/flatpak/detect", function (data)
+    $.getJSON("/api/flatpak/app", function (response)
     {
-        if (!data.available)
+        if (!response.result || !response.payload)
         {
             console.log('Flatpak not available on this system');
             return;
         }
 
-        // Enable and update stable option
-        if (data.stable.installed)
-        {
-            $("#flatpak-stable-option").prop('disabled', false);
-            $("#stable-version").text('v' + data.stable.version);
-        }
-        else
-        {
-            $("#stable-version").text('not installed');
-        }
+        // Reset options first
+        $("#flatpak-stable-option").prop('disabled', true);
+        $("#flatpak-master-option").prop('disabled', true);
+        $("#stable-version").text('not installed');
+        $("#master-version").text('not installed');
 
-        // Enable and update master option
-        if (data.master.installed)
+        // Process each flatpak in the payload array
+        response.payload.forEach(function (flatpak)
         {
-            $("#flatpak-master-option").prop('disabled', false);
-            $("#master-version").text('v' + data.master.version);
-        }
-        else
-        {
-            $("#master-version").text('not installed');
-        }
+            if (flatpak.application_id === "org.kde.kstars")
+            {
+                // Check branch to determine stable vs master
+                if (flatpak.branch === "stable")
+                {
+                    $("#flatpak-stable-option").prop('disabled', false);
+                    $("#stable-version").text('v' + flatpak.version);
+                }
+                else if (flatpak.branch === "master")
+                {
+                    $("#flatpak-master-option").prop('disabled', false);
+                    $("#master-version").text('v' + flatpak.version);
+                }
+            }
+        });
     }).fail(function ()
     {
         console.error('Failed to detect flatpaks');
@@ -551,3 +642,66 @@ function detectFlatpaks()
         $("#master-version").text('detection failed');
     });
 }
+
+// Handle driver source change
+function onDriverSourceChange()
+{
+    var source = $("#driver_source").val();
+    console.log("Driver source changed to: " + source);
+
+    // Step 1: Capture currently selected drivers before switching
+    var selectedDrivers = [];
+    $("#drivers_list :selected").each(function (i, sel)
+    {
+        selectedDrivers.push($(sel).text());
+    });
+    console.log("Currently selected drivers:", selectedDrivers);
+
+    // Save the current profile to update driver_source
+    saveProfileInfo();
+
+    // Step 2: Reload drivers from the new source
+    loadDriversFromSource(source, function ()
+    {
+        // Step 3: Restore selections for drivers that exist in new source
+        var restoredCount = 0;
+        var notFoundDrivers = [];
+
+        selectedDrivers.forEach(function (driverLabel)
+        {
+            var selector = "#drivers_list [value='" + driverLabel + "']";
+            if ($(selector).length > 0)
+            {
+                $(selector).prop('selected', true);
+                restoredCount++;
+            }
+            else
+            {
+                notFoundDrivers.push(driverLabel);
+            }
+        });
+
+        $("#drivers_list").selectpicker('refresh');
+
+        // Step 4: Show appropriate notification
+        var message = 'Driver source changed to ' + source + '. ';
+        if (restoredCount > 0)
+        {
+            message += restoredCount + ' driver(s) preserved.';
+            if (notFoundDrivers.length > 0)
+            {
+                message += ' Note: ' + notFoundDrivers.length + ' driver(s) not available in this source: ' + notFoundDrivers.join(', ') + '.';
+            }
+
+            // Auto-save the profile with preserved selections
+            var profileName = $("#profiles option:selected").text();
+            saveProfileDrivers(profileName, true);
+        }
+        else
+        {
+            message += 'Please select drivers for this profile.';
+        }
+
+        $("#notify_message").html('<br/><div class="alert alert-info"><a href="#" class="close" data-dismiss="alert" aria-label="close">&times;</a>' + message + '</div>');
+    });
+}
